<?php
/**
 * In Login page the first Login form submit triggers this ajax call.
 * This function checks if the user has activated the Two Factor Authentication and generate OTP if needed.
 */
function twofa_check_authentication()
{
    try {
        $processes = new TwoFAProcesses();
        $authenticatedUserID = $processes->getUserIdIfExists($_POST['username']);
        if ($processes->userHasTwoFAEnabled($authenticatedUserID)) {
            //The User has Two Factor authentication enabled
            $api = new TwoFAApi();
            $destination = $processes->getUserFullMobileNumber($authenticatedUserID);
            $otp = $api->generateOTP($destination);
            $processes->saveUserOTP($authenticatedUserID, $otp);
            $api->sendSMS($destination, $otp);
            echo json_encode(array(
                'status' => 'success',
                'resultCode' => 'OTP_AUTH_REQUIRED',
                'message' => 'User have Two Factor authentication enabled. OTP is generated.',
                'data' => array(
                    'destination' => $destination,
                    'otp' => $otp
                )
            ));
        } else {
            //User doesn't have Two Factor authentication enabled
            echo json_encode(array(
                'status' => 'success',
                'resultCode' => 'NO_AUTH_PROCEED',
                'message' => 'User doesn\'t have Two Factor authentication enabled'
            ));
        }
    } catch (Exception $e) {
        //Exception Occurred
        echo json_encode(array(
            'status' => 'failed',
            'message' => $e->getMessage()
        ));
    }
    wp_die();
}

add_action('wp_ajax_twofa_check_authentication', 'twofa_check_authentication');
add_action('wp_ajax_nopriv_twofa_check_authentication', 'twofa_check_authentication');

/**
 * This function is called on the second ajax call from Login form where the OTP have to be verified.
 */
function twofa_verify_otp()
{
    try {
        $processes = new TwoFAProcesses();
        $authenticatedUserID = $processes->getUserIdIfExists($_POST['username']);
        if ($processes->userHasTwoFAEnabled($authenticatedUserID)) {
            //The User has Two Factor Authentication enabled.
            $api = new TwoFAApi();
            $destination = $processes->getUserFullMobileNumber($authenticatedUserID);
            $savedOTP = $processes->getUserOTP($authenticatedUserID);
            if ($_POST['otp'] === $savedOTP && $api->verifyOTP($destination, $_POST['otp'])) {
                //From this point the OTP is not valid anymore if checked with verifyOTP function.
                //So we need a security code to validate once more after the user has logged in.
                echo json_encode(array(
                    'status' => 'success',
                    'resultCode' => 'OTP_AUTH_PROCEED',
                    'message' => 'OTP is verified, proceed to login.',
                    'securityCode' => $processes->generateOTPSecurityCode($_POST['otp'])
                ));
            } else {
                echo json_encode(array(
                    'status' => 'failed',
                    'message' => 'OTP Verification failed. OTP is invalid or Expired. Please try again.'
                ));
            }
        } else {
            //User doesn't have Two Factor authentication enabled.
            //This shouldn't happen so shh... Just let them login if they have the right password.
            echo json_encode(array(
                'status' => 'success',
                'resultCode' => 'NO_AUTH_PROCEED',
                'message' => 'No OTP Verification required.'
            ));
        }
    } catch (Exception $e) {
        //Exception Occurred
        echo json_encode(array(
            'status' => 'failed',
            'message' => $e->getMessage()
        ));
    }
    wp_die();
}

add_action('wp_ajax_twofa_verify_otp', 'twofa_verify_otp');
add_action('wp_ajax_nopriv_twofa_verify_otp', 'twofa_verify_otp');

function twofa_verify_phone()
{
    try {
        $api = new TwoFAApi();
        $destination = $_POST['destination'];
        $otp = $api->generateOTP($destination);
        $api->sendSMS($destination, $otp);
        echo json_encode(array(
            'status' => 'success',
            'resultCode' => 'OTP_AUTH_REQUIRED',
            'message' => 'User have Two Factor authentication enabled. OTP is generated.',
            'data' => array(
                'destination' => $destination,
                'otp' => $otp
            )
        ));
    } catch (Exception $e) {
        //Exception Occurred
        echo json_encode(array(
            'status' => 'failed',
            'message' => $e->getMessage()
        ));
    }
    wp_die();
}

add_action('wp_ajax_twofa_verify_phone', 'twofa_verify_phone');
add_action('wp_ajax_nopriv_twofa_verify_phone', 'twofa_verify_phone');