<?php

class TwoFAProcesses
{
    const NonceOTPPrepend = 'twofa-otp-nonce-';

    public function getUserIdIfExists($userName)
    {
        $user = get_user_by('login', $userName);
        if ($user instanceof WP_User) {
            return $user->ID;
        } else {
            $user = get_user_by('email', $userName);
            if ($user instanceof WP_User) {
                return $user->ID;
            }
        }
        throw new Exception('Invalid Username/Email Provided.');
    }

    public function userHasTwoFAEnabled($userID)
    {
        $twofa_activated = get_user_meta($userID, 'twofa_activated');
        return is_array($twofa_activated) && isset($twofa_activated[0]) ? true : false;
    }

    public function getUserCountry($userID)
    {
        $twofa_country = get_user_meta($userID, 'twofa_county');
        return is_array($twofa_country) && isset($twofa_country[0]) ? $twofa_country[0] : false;
    }

    public function getUserMobile($userID)
    {
        $twofa_mobile = get_user_meta($userID, 'twofa_mobile');
        return is_array($twofa_mobile) && isset($twofa_mobile[0]) ? $twofa_mobile[0] : '';
    }

    public function getUserFullMobileNumber($userID)
    {
        return $this->getUserCountry($userID) . $this->getUserMobile($userID);
    }

    public function saveUserOTP($userID, $otp)
    {
        delete_user_meta($userID, 'twofa_otp');
        add_user_meta($userID, 'twofa_otp', $otp, true);
    }

    public function getUserOTP($userID, $delete = false)
    {
        $twofa_otp = get_user_meta($userID, 'twofa_otp');
        if ($delete) {
            delete_user_meta($userID, 'twofa_otp');
        }
        return is_array($twofa_otp) && isset($twofa_otp[0]) ? $twofa_otp[0] : false;
    }

    public function generateOTPSecurityCode($otp)
    {
        //I didn't want to do this. I wanted to use nonce but it's not working as I expected.
        return md5(self::NonceOTPPrepend . $otp);
    }

    public function verifyOTPSecurityCode($code, $otp)
    {
        //This is a quickie and is subject to change in future.
        $recreate = md5(self::NonceOTPPrepend . $otp);
        if ($code === $recreate) {
            return true;
        } else {
            return false;
        }
    }
}