<?php

/**
 * Class TwoFASettings
 */
class TwoFASettings
{
    const GROUP = 'twofa-settings-group';
    const PAGE = 'twofasettings';
    const SECTION = 'twofa-settings';

    protected $settings;

    /**
     * TwoFASettings constructor.
     */
    function __construct()
    {
        $this->settings = (array)get_option('twofa-settings');
    }

    function registerSettings()
    {
        /*
	   * http://codex.wordpress.org/Function_Reference/register_setting
	   * register_setting( $option_group, $option_name, $sanitize_callback );
	   * The second argument ($option_name) is the option name. It’s the one we use with functions like get_option() and update_option()
	   * */
        # With input validation:
        # register_setting( 'my-settings-group', 'my-plugin-settings', 'my_settings_validate_and_sanitize' );
        register_setting('twofa-settings-group', self::SECTION, array($this, 'twofa_validate_callback'));

        /*
   * http://codex.wordpress.org/Function_Reference/add_settings_section
   * add_settings_section( $id, $title, $callback, $page );
   * */
        add_settings_section(self::SECTION, '', array($this, 'twofa_settings_callback'), self::PAGE);


        /*
         * http://codex.wordpress.org/Function_Reference/add_settings_field
         * add_settings_field( $id, $title, $callback, $page, $section, $args );
         * */
        add_settings_field('field-twofa-user-roles', $this->getFieldTitle('field-twofa-user-roles'), array($this, 'twofa_user_roles'), self::PAGE, self::SECTION);

        add_settings_field('field-twofa-credentials', $this->getFieldTitle('field-twofa-credentials'), array($this, 'twofa_credentials'), self::PAGE, self::SECTION);
    }

    /**
     * @param $input
     * @return mixed
     */
    function twofa_validate_callback($input)
    {
        return $input;
    }

    function twofa_settings_callback()
    {
        _e('Two Factor Authentication settings', self::PAGE);
    }

    function twofa_user_roles()
    {
        ?>
        <!-- Code in class.TwoFASettings.php file-->
        <link rel="stylesheet" href="<?php echo TwoFA_PLUGIN_URL; ?>css/style.php">
        <?php
        $roles = self::get_user_roles();
        $field = 'allowed_roles';
        echo '<table>';
        foreach ($roles as $role) {
            $role_slug = strtolower($role['name']);
            $value = esc_attr($this->settings[$field]);
            if (isset($this->settings[$field]) && in_array($role_slug, $this->settings[$field])) {
                $checked = 'checked="checked"';
            } else {
                $checked = '';
            }
            echo "<tr>";
            echo "<td><input type='checkbox' name='twofa-settings[$field][]' value='" . $role_slug . "' " . $checked . " /></td>";
            echo "<td>" . $role['name'] . "</td>";
            echo "</tr>";
        }
        echo '</table>';
    }

    function twofa_credentials()
    {
        echo "<table style='text-align: right'>";
        echo "<tr>";
        echo "<td>User</td>";
        echo "<td><input name='twofa-settings[credentials][user]' value='" . esc_attr($this->settings['credentials']['user']) . "' style='width:250px;'/></td>";
        echo "</tr>";
        echo "<tr>";
        echo "<td>Password</td>";
        echo "<td><input name='twofa-settings[credentials][password]' value='" . esc_attr($this->settings['credentials']['password']) . "' style='width:250px;'/></td>";
        echo "</tr>";
        echo "</table>";
    }

    private function getFieldTitle($field)
    {
        switch ($field) {
            case 'field-twofa-user-roles':
                return __('User Roles', self::PAGE) . '<div><small>Only selected user roles will have the Two Factor Authentication enabled.</small></div>';
                break;
            case 'field-twofa-credentials':
                return __('Afilnet Credentials', self::PAGE) . '<div><small>Details to access the Afilnet API.</small></div>';
                break;
            default:
                return $field;
        }
    }

    static function get_user_roles()
    {
        global $wp_roles;

        $all_roles = $wp_roles->roles;
        $editable_roles = apply_filters('editable_roles', $all_roles);

        return $editable_roles;
    }

    static function get_settings($field = null)
    {
        $settings = (array)get_option(self::SECTION);
        if ($field === null) {
            return $settings;
        } else {
            if (isset($settings[$field])) {
                return $settings[$field];
            } else {
                return false;
            }
        }
    }
}