jQuery(document).ready(function ($) {
    var TwoFALoginAuth = {
        loginForm: null,
        byPassSubmitCheck: false,
        submitToVerifyOTP: false,
        registeredNumber: false,
        flashInterval: null,
        init: function () {
            var root = this;
            this.loginForm = $('#wp-submit').parents('form[name!="resetpassform"]:first');
            this.loginForm.on('submit', function () {
                if (root.byPassSubmitCheck) {
                    //This means the form needs no further check to proceed to login.
                    return true;
                } else if (root.submitToVerifyOTP) {
                    //This means this is the submit to verify OTP.
                    root.verifyOTP();
                    return false;
                } else {
                    //The initial submit.
                    root.onSubmit();
                    return false;
                }
            });

            //SetUp Loader during ajax
            this.initLoader();
        },
        initLoader: function () {
            var html = '<div class="spinner_wrapper">' +
                '<div class="spinner_container"> ' +
                '<div class="spinner">' +
                '  <div class="double-bounce1"></div>' +
                '  <div class="double-bounce2"></div>' +
                '</div>' +
                '</div>' +
                '</div>';
            var css = '<style>' +
                '.spinner_wrapper{' +
                'position: absolute;' +
                'height: 100%;' +
                'width: 100%;' +
                'background-color: rgba(0, 0, 0, 0.45);' +
                'display: none;' +
                '}' +
                '.spinner_container{' +
                'display: table-cell;' +
                'vertical-align: middle;' +
                '}' +
                '.spinner {' +
                '  width: 40px;' +
                '  height: 40px;' +
                '' +
                '  position: relative;' +
                '  margin: 100px auto;' +
                '}' +
                '' +
                '.double-bounce1, .double-bounce2 {' +
                '  width: 100%;' +
                '  height: 100%;' +
                '  border-radius: 50%;' +
                '  background-color: #fff;' +
                '  opacity: 0.6;' +
                '  position: absolute;' +
                '  top: 0;' +
                '  left: 0;' +
                '  ' +
                '  -webkit-animation: sk-bounce 2.0s infinite ease-in-out;' +
                '  animation: sk-bounce 2.0s infinite ease-in-out;' +
                '}' +
                '' +
                '.double-bounce2 {' +
                '  -webkit-animation-delay: -1.0s;' +
                '  animation-delay: -1.0s;' +
                '}' +
                '' +
                '@-webkit-keyframes sk-bounce {' +
                '  0%, 100% { -webkit-transform: scale(0.0) }' +
                '  50% { -webkit-transform: scale(1.0) }' +
                '}' +
                '' +
                '@keyframes sk-bounce {' +
                '  0%, 100% { ' +
                '    transform: scale(0.0);' +
                '    -webkit-transform: scale(0.0);' +
                '  } 50% { ' +
                '    transform: scale(1.0);' +
                '    -webkit-transform: scale(1.0);' +
                '  }' +
                '}</style>';
            $('body').prepend(css + html);
        },
        toggleLoader: function (toggle) {
            if (toggle === 'show') {
                $('.spinner_wrapper').css({
                    display: 'table'
                });
            } else if (toggle === 'hide') {
                $('.spinner_wrapper').hide();
            }
        },
        onSubmit: function () {
            console.log('TwoFA: Form Submitted');
            var root = this;
            //Ajax to check if user have Two Factor Authentication and Generate OTP.
            var data = {
                'action': 'twofa_check_authentication',
                'username': $('#user_login').val()
            };
            this.toggleLoader('show');
            $.post(TwoFAAjax.ajaxURL, data, function (response) {
                root.processResponse(response);
            });
        },
        getOTPFormHtml: function () {
            var html = '<p class="twofa_otp_wrapper">' +
                '<label for="user_login">Enter OTP<br>' +
                '<input type="text" name="otp" id="user_otp" aria-describedby="login_error" class="input" value="" size="20"></label>' +
                'An OTP Has been send to your registered number ' + this.registeredNumber +
                '</p>'
            return html;
        },
        showOTPForm: function () {
            console.log('TwoFA : Showing OTP Form');
            //Hide both Username and Password fields
            $('#user_login').parents('p').hide();
            $('#user_pass').parents('p').hide();

            //Show OTP Field
            this.loginForm.prepend(this.getOTPFormHtml());
        },
        verifyOTP: function () {
            console.log('TwoFA : Verifying OTP');
            var root = this;
            //Ajax to check if user have Two Factor Authentication and Generate OTP.
            var data = {
                'action': 'twofa_verify_otp',
                'username': $('#user_login').val(),
                'otp': $('#user_otp').val()
            };
            this.toggleLoader('show');
            $.post(TwoFAAjax.ajaxURL, data, function (response) {
                root.processResponse(response);
            });
        },
        processResponse: function (response) {
            if (response) {
                response = JSON.parse(response);
                console.log(response);
                if (response.status === 'success') {
                    switch (response.resultCode) {
                        case 'OTP_AUTH_REQUIRED':
                            this.submitToVerifyOTP = true;
                            this.registeredNumber = response.data.destination;
                            this.showOTPForm();
                            break;
                        case 'OTP_AUTH_PROCEED':
                            this.loginForm.append('<input type="hidden" name="otp-security-code" value="' + response.securityCode + '">');
                        case 'NO_AUTH_PROCEED':
                            //We are just avoiding further ajax calls from submitting the form.
                            this.byPassSubmitCheck = true;
                            this.loginForm.submit();
                            break;
                        default:
                            this.resetForm();
                            this.displayError('Fatal Error : Obtained an Invalid Success.');
                    }
                } else {
                    //Process Failed. Display the error message.
                    if (typeof response.message !== 'undefined' && response.message.length > 0) {
                        this.displayError(response.message);
                    } else {
                        this.displayError('Fatal Error : Something went wrong. Please try again.');
                    }
                    this.resetForm();
                }
            }
            this.toggleLoader('hide');
        },
        displayError: function (message) {
            console.warn(message);
            $html = '<div id="login_error" class="twofa_custom">' +
                '<strong>ERROR</strong>: ' + message +
                '</div>';
            $('#login_error.twofa_custom').remove();
            this.loginForm.before($html);
        },
        resetForm: function () {
            this.submitToVerifyOTP = false;
            this.byPassSubmitCheck = false;
            $('.twofa_otp_wrapper').remove();
        }
    };

    TwoFALoginAuth.init();
});