jQuery(document).ready(function ($) {
    //Function to verify the phone number before the actual Two Factor Authentication is activated.
    var TwoFAManage = {
        formObject: $('#twofa-manage-form'),
        formAccepted: false,
        twofaActivateCheckbox: $('#twofa-activated'),
        init: function () {
            var root = this;
            this.updateForm();
            //Setup Triggers
            this.twofaActivateCheckbox.change(function () {
                root.updateForm();
            });
            this.formObject.on('submit', function () {
                root.processForm();
                return root.formAccepted;
            });
            this.initSpinner();
        },
        updateForm: function () {
            var twofaActivated = this.twofaActivateCheckbox.prop('checked');
            if (twofaActivated) {
                $('.twofa-manage-row').show();
            } else {
                $('.twofa-manage-row').hide();
            }
        },
        processForm: function () {
            if (!this.formAccepted) {
                if (this.twofaActivateCheckbox.prop('checked')) {
                    var root = this;
                    var destination = $('#twofa-country-code').val() + $('#twofa-phone-number').val();
                    //Ajax to Generate OTP for verifying the phone number.
                    var data = {
                        'action': 'twofa_verify_phone',
                        'destination': destination
                    };
                    this.showSpinner();
                    $.post('admin-ajax.php', data, function (response) {
                        root.processResponse(response);
                    });
                } else {
                    //Two Factor Authentication checkbox is unchecked.
                    //So no need to verify phone
                    this.formAccepted = true;
                }
            }

        },
        processResponse: function (response) {
            if (response) {
                response = JSON.parse(response);
                console.log(response);
                if (response.status === 'success') {
                    switch (response.resultCode) {
                        case 'OTP_AUTH_REQUIRED':
                            this.formAccepted = true;
                            this.showOTPForm();
                            break;
                        default:
                            alert('Fatal Error : Obtained an Invalid Success.');
                    }
                } else {
                    //Process Failed. Display the error message.
                    if (typeof response.message !== 'undefined' && response.message.length > 0) {
                        alert(response.message);
                    } else {
                        alert('Fatal Error : Something went wrong. Please try again.');
                    }
                }
            }
            this.hideSpinner();
        },
        showOTPForm: function () {
            var html = '<tr>' +
                '           <td>' +
                '               <strong>Enter OTP</strong><br>' +
                '               <small>Enter the OTP send to the Number provided<br> for verification and activate<br>' +
                '               <strong>Two Factor Authentication.</strong></small>' +
                '           </td>' +
                '           <td>' +
                '               <input name="twofa_verify_otp" class="regular-text" placeholder="OTP">' +
                '           </td>' +
                '       </tr>';

            $('table#twofa-manage-table tr').css({
                visibility: 'hidden',
                position: 'absolute'
            });
            $('#twofa-manage-table').append(html);
        },
        initSpinner: function () {
            var html = '<div class="spinner" style="float: none;"></div>';
            this.formObject.append(html);
        },
        showSpinner: function () {
            $('.spinner').css({
                visibility: 'unset'
            });
        },
        hideSpinner: function () {
            $('.spinner').css({
                visibility: 'hidden'
            });
        }
    };
    TwoFAManage.init();
});